/*
 * DateComparator.java
 * Part of the B'am library - a collection of general-purpose utility classes.
 * 
 * Copyright (C)2006 Stephen Battey
 * Contact: resource@lucidviews.net
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

package net.lucidviews.util;

import java.util.Date;
import java.util.Comparator;


/**
 * A <code>Comparator</code> that compares dates.
 * <br />
 * <br />
 * @author Stephen Battey
 * @version $Revision: 1.1 $
 * @since 1.0
 * @see java.util.Comparator
 * @see net.lucidviews.util.io.FileDateComparator
 */
public class DateComparator implements Comparator
{
	
	/*
	 * Exported constants
	 */
	
	/** A <code>DateComparator</code> that puts dates into chronological order.
	 *  */
	public static final DateComparator CHRONOLOGICAL = new DateComparator( false );
	
	/** A <code>DateComparator</code> that puts dates into reverse
	 *  chronological order. */
	public static final DateComparator REVERSE_CHRONOLOGICAL = new DateComparator( true );
	
	
	/*
	 * Instance variables
	 */
	
	/** The number comparator that will be used to compare UTC time. */
	protected NumberComparator _timestampComparator;
	
	
	/*
	 * Constructors
	 */
	
	/**
	 * The default constructor.
	 * <br />
	 * Creates a comparator that will sort dates into chronological order.
	 */
	protected DateComparator()
	{
		this( false );
	}
	
	/**
	 * Create a comparator that will sort dates.
	 * <br />
	 * <br />
	 * @param reverse if set to <code>false</code> dates will be put in
	 *                chronological order; if set to <code>true</code> dates
	 *                will be put in reverse chronological order
	 */
	protected DateComparator( boolean reverse )
	{
		_timestampComparator = (reverse) ? NumberComparator.DESCENDING
		                                 : NumberComparator.ASCENDING;
	}
	
	
	/*
	 * Methods
	 */
	
	/**
	 * Compares two <code>Date</code>s for order. Returns a negative integer,
	 * zero, or a positive integer as the first date is less than, equal
	 * to, or greater than the second.
	 * <br />
	 * <br />
	 * This implementation ensures that <tt>sgn(compare(x, y)) ==
	 * -sgn(compare(y, x))</tt> for all <tt>x</tt> and <tt>y</tt>. (This
	 * implies that <tt>compare(x, y)</tt> must throw an exception if and only
	 * if <tt>compare(y, x)</tt> throws an exception.)
	 * <br />
	 * <br />
	 * This implementation also ensures that the relation is transitive:
	 * <tt>((compare(x, y)&gt;0) &amp;&amp; (compare(y, z)&gt;0))</tt> implies
	 * <tt>compare(x, z)&gt;0</tt>.
	 * <br />
	 * <br />
	 * This implementation also ensures that <tt>compare(x, y)==0</tt>
	 * implies that <tt>sgn(compare(x, z))==sgn(compare(y, z))</tt> for all
	 * <tt>z</tt>.
	 * <br />
	 * <br />
	 * Finally, this implementation also ensures that
	 * <tt>(compare(x, y)==0) == (x.equals(y))</tt>.
	 * <br />
	 * <br />
	 * @param date1 the first date to be compared
	 * @param date2 the second date to be compared
	 * @return a negative integer, zero, or a positive integer as the
	 * 	       first date is less than, equal to, or greater than the
	 *	       second
	 */
	public int compare( Date date1, Date date2 )
	{
		return compare( date1.getTime(), date2.getTime() );
	}
	
	/**
	 * Compares two timestamps for order. Returns a negative integer,
	 * zero, or a positive integer as the first number is less than, equal
	 * to, or greater than the second.
	 * <br />
	 * <br />
	 * This implementation ensures that <tt>sgn(compare(x, y)) ==
	 * -sgn(compare(y, x))</tt> for all <tt>x</tt> and <tt>y</tt>. (This
	 * implies that <tt>compare(x, y)</tt> must throw an exception if and only
	 * if <tt>compare(y, x)</tt> throws an exception.)
	 * <br />
	 * <br />
	 * This implementation also ensures that the relation is transitive:
	 * <tt>((compare(x, y)&gt;0) &amp;&amp; (compare(y, z)&gt;0))</tt> implies
	 * <tt>compare(x, z)&gt;0</tt>.
	 * <br />
	 * <br />
	 * This implementation also ensures that <tt>compare(x, y)==0</tt>
	 * implies that <tt>sgn(compare(x, z))==sgn(compare(y, z))</tt> for all
	 * <tt>z</tt>.
	 * <br />
	 * <br />
	 * Finally, this implementation also ensures that
	 * <tt>(compare(x, y)==0) == (x.equals(y))</tt>.
	 * <br />
	 * <br />
	 * @param timestamp1 the first timestamp to be compared
	 * @param timestamp2 the second timestamp to be compared
	 * @return a negative integer, zero, or a positive integer as the
	 * 	       first timestamp is less than, equal to, or greater than the
	 *	       second
	 */
	public int compare( long timestamp1, long timestamp2 )
	{
		return _timestampComparator.compare( timestamp1, timestamp2 );
	}
	
	
	/*
	 * Implementation of Comparator interface
	 */
	
	public int compare( Object o1, Object o2 )
	{
		return compare( (Date)o1, (Date)o2 );
	}
	
	public boolean equals( Object obj )
	{
		if (obj instanceof DateComparator)
		{
			DateComparator other = (DateComparator)obj;
			
			// equal if same ordering
			return _timestampComparator.equals( other._timestampComparator );
		}
		else
		{
			// not a DateComparator, so can't be equal
			return false;
		}
	}
	
}


/*
 * Change History
 *
 * $Log: DateComparator.java,v $
 * Revision 1.1  2006/09/13 20:09:05  Steve
 * Initial revision.
 *
 */