/*
 * FileDateComparator.java
 * Part of the B'am library - a collection of general-purpose utility classes.
 * 
 * Copyright (C)2006 Stephen Battey
 * Contact: resource@lucidviews.net
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

package net.lucidviews.util.io;

import net.lucidviews.util.DateComparator;

import java.io.File;


/**
 * A <code>FileComparator</code> that compares files using their last-modified
 * timestamp.
 * <br />
 * By default this comparator places files in chronological order. I.e: more
 * recent files would be pushed to the end of a list and older files would
 * rise to the top.
 * <br />
 * This logic can be switched by constructing an instance with a
 * <code>DateComparator</code> that sorts dates in a reverse chronological
 * order.
 * <br />
 * <br />
 * @author Stephen Battey
 * @version $Revision: 1.1 $
 * @since 1.0
 * @see java.util.Comparator
 * @see net.lucidviews.util.io.FileComparator
 * @see net.lucidviews.util.DateComparator
 */
public class FileDateComparator extends AbstractFileComparator
{
	
	/*
	 * Exported constants
	 */
	
	/** A <code>FileDateComparator</code> that puts files into chronological
	 *  order, based on their (last-modified) timestamp. */
	public static final FileDateComparator CHRONOLOGICAL = new FileDateComparator( DateComparator.CHRONOLOGICAL );
	
	/** A <code>FileDateComparator</code> that puts files into reverse
	 *  chronological order, based on their (last-modified) timestamp. */
	public static final FileDateComparator REVERSE_CHRONOLOGICAL = new FileDateComparator( DateComparator.REVERSE_CHRONOLOGICAL );
	
	
	/*
	 * Instance variables
	 */
	
	/** The comparator used to compare file timestamps. */
	protected DateComparator _dateComparator;
	
	
	/*
	 * Constructors
	 */
	
	/**
	 * The default constructor.
	 * <br />
	 * A comparator that will put files in chronological order according to
	 * their (last modified) timestamp.
	 */
	protected FileDateComparator()
	{
		this( DateComparator.CHRONOLOGICAL );
	}
	
	/**
	 * Create a comparator that will order files according to their (last
	 * modified) timestamp.
	 * <br />
	 * <br />
	 * @param dateComparator the comparator to use to compare file timestamps
	 */
	protected FileDateComparator( DateComparator dateComparator )
	{
		_dateComparator = dateComparator;
	}
	
	
	/*
	 * Implementation of FileComparator interface
	 */
	
	public int compare( File file1, File file2 )
	{
		return _dateComparator.compare( file1.lastModified(), file2.lastModified() );
	}
	
	
	/*
	 * Implementation of Comparator interface
	 */
	
	public boolean equals( Object obj )
	{
		if (obj instanceof FileDateComparator)
		{
			FileDateComparator other = (FileDateComparator)obj;
			
			// equal if same ordering
			return this._dateComparator.equals( other._dateComparator );
		}
		else
		{
			// not a FileDateComparator, so can't be equal
			return false;
		}
	}
	
}


/*
 * Change History
 *
 * $Log: FileDateComparator.java,v $
 * Revision 1.1  2006/09/13 20:09:30  Steve
 * Initial revision.
 *
 */