/*
 * Files.java
 * Part of the B'am library - a collection of general-purpose utility classes.
 * 
 * Copyright (C)2005, 2006 Stephen Battey
 * Contact: resource@lucidviews.net
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

package net.lucidviews.util.io;

import java.io.InputStream;
import java.io.OutputStream;
import java.io.IOException;


/**
 * A collection of methods for manipualting data <code>Stream</code>s.
 * <br />
 * <br />
 * @author Stephen Battey
 * @version $Revision: 1.2 $
 * @since 1.0
 */
public class Streams
{

	/*
	 * Local constants
	 */

	/** the default buffer size used to pipe data across I/O streams */
	protected static final int DEFAULT_BUFFER_SIZE = 5120; /* 5K */


	/*
	 * Constructor(s)
	 */

	/**
	 * Suppresses default constructor, ensuring non-instantiability.
	 */
	protected Streams()
	{
	}


	/*
	 * Static methods
	 */

	/**
	 * Pipe data across I/O streams.
	 * <br />
	 * The transfer will take place using a data-buffer of default size.
	 * <br />
	 * <br />
	 * @param in the input stream
	 * @param out the output stream
	 * @throws IOException error reading or writing data
	 */
	public static void pipe( InputStream in, OutputStream out ) throws IOException
	{
		pipe( in, out, DEFAULT_BUFFER_SIZE );
	}

	/**
	 * Pipe data across I/O streams.
	 * <br />
	 * The transfer will take place using a data-buffer of the specified size.
	 * <br />
	 * <br />
	 * @param in the input stream
	 * @param out the output stream
	 * @param bufferSize the size of the data buffer
	 * @throws IOException error reading or writing data
	 */
	public static void pipe( InputStream in, OutputStream out, int bufferSize ) throws IOException
	{
		byte[] buf = new byte[bufferSize];
		int bytesRead = in.read( buf );
		while (bytesRead > -1)
		{
			out.write( buf, 0, bytesRead );
			bytesRead = in.read( buf );
		}
	}

	/**
	 * Close an input stream.
	 * <br />
	 * If the stream is <code>null</code> then this method has no action. Any
	 * exception caused by closing the stream is silently handled.
	 * <br />
	 * <br />
	 * @param stream the stream to be closed
	 */
	public static void close( InputStream stream )
	{
		if (stream != null)
		{
			try
			{
				stream.close();
			}
			catch (Throwable e)
			{
			}
		}
	}

	/**
	 * Close an output stream.
	 * <br />
	 * If the stream is <code>null</code> then this method has no action. Any
	 * exception caused by closing the stream is silently handled.
	 * <br />
	 * <br />
	 * @param stream the stream to be closed
	 */
	public static void close( OutputStream stream )
	{
		if (stream != null)
		{
			try
			{
				stream.close();
			}
			catch (Throwable e)
			{
			}
		}
	}

	/**
	 * Compare the content of two streams to determine if the data in both
	 * streams is the same.
	 * <br />
	 * The comparison will be performed using a data-buffer of the specified
	 * size.
	 * <br />
	 * <br />
	 * Once the comparison is completed, both streams will be closed.
	 * <br />
	 * <br />
	 * @param stream1 a stream to be compared to the second stream
	 * @param stream2 a stream to be compared to the first stream
	 * @return <code>true</code> if the binary content of both streams match
	 *         exactly
	 * @throws IOException error reading data from the streams
	 */
	public static boolean compare( InputStream stream1, InputStream stream2 ) throws IOException
	{
		return compare( stream1, stream2, DEFAULT_BUFFER_SIZE );
	}

	/**
	 * Compare the content of two streams to determine if the data in both
	 * streams is the same.
	 * <br />
	 * <br />
	 * The comparison will be performed using a data-buffer of the specified
	 * size.
	 * <br />
	 * <br />
	 * Once the comparison is completed, both streams will be closed.
	 * <br />
	 * <br />
	 * @param stream1 a stream to be compared to the second stream
	 * @param stream2 a stream to be compared to the first stream
	 * @param bufferSize the size of the data buffer used when comparing binary
	 *                   data
	 * @return <code>true</code> if the binary content of both streams match
	 *         exactly
	 * @throws IOException error reading data from the streams
	 */
	public static boolean compare( InputStream stream1,
								   InputStream stream2,
	                               int bufferSize ) throws IOException
	{
		try
		{
			byte[] buf1 = new byte[bufferSize];
			byte[] buf2 = new byte[bufferSize];

			int bytesRead1 = stream1.read( buf1 );
			int bytesRead2 = stream2.read( buf2 );

			if (bytesRead1 != bytesRead2)
				/* content is of different length and therefore cannot be equal */
				return false;

			while (   (bytesRead1 > -1)
				   && (bytesRead2 > -1)  )
			{
				for (int i=0; i<bytesRead1; i++)
				{
					if (buf1[i] != buf2[i])
						/* bytes do not match */
						return false;
				}

				bytesRead1 = stream1.read( buf1 );
				bytesRead2 = stream2.read( buf2 );

				if (bytesRead1 != bytesRead2)
					/* content is of different length and therefore cannot be equal */
					return false;
			}

			/* we have come to the end of both streams with all bytes equal */
			return true;
		}
		finally
		{
			close( stream1 );
			close( stream2 );
		}
	}

}


/*
 * Change History
 *
 * $Log: Streams.java,v $
 * Revision 1.2  2006/09/06 20:52:02  Steve
 * Added legal disclaimer.
 *
 * Revision 1.1  2006/04/12 16:15:35  Steve
 * Initial revision.
 *
 */
